@@ -10,6 +10,7 @@
 #include "../utils_p.h"
 #include "kfilewidgets_debug.h"
 #include "knameandurlinputdialog.h"
+#include "ui_knewfilemenu_newfiledialog.h"
 
 #include <kdirnotify.h>
 #include <kio/copyjob.h>
@@ -25,15 +26,19 @@
 #include <kprotocolmanager.h>
 #include <kurifilter.h>
 
+#include <KCollapsibleGroupBox>
 #include <KConfigGroup>
 #include <KDesktopFile>
 #include <KDirOperator>
 #include <KDirWatch>
 #include <KFileUtils>
+#include <KIconDialog>
+#include <KIconLoader>
 #include <KJobWidgets>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KMessageWidget>
+#include <KSharedConfig>
 #include <KShell>
 
 #include <QActionGroup>
@@ -41,6 +46,7 @@
 #include <QDialog>
 #include <QDialogButtonBox>
 #include <QDir>
+#include <QFontDatabase>
 #include <QLabel>
 #include <QLineEdit>
 #include <QList>
@@ -76,6 +82,22 @@ static QString expandTilde(const QString &name, bool isfile = false)
     return !expandedName.isEmpty() ? expandedName : name;
 }
 
+static bool isDefaultFolderIcon(const QString &iconName)
+{
+    return iconName.isEmpty() || iconName == QLatin1String("folder") || iconName == QLatin1String("inode-directory");
+}
+
+static bool canPickFolderIcon(const QUrl &url)
+{
+    // TODO mostLocalUrl? But that would mean stat'ing when opening the dialog as opposed to only when accepting.
+    return url.isLocalFile() || KProtocolInfo::protocolClass(url.scheme()) == QLatin1String(":local");
+}
+
+static KConfigGroup stateConfig()
+{
+    return KConfigGroup(KSharedConfig::openStateConfig(QStringLiteral("kiostaterc")), QStringLiteral("New File Menu"));
+}
+
 // Singleton, with data shared by all KNewFileMenu instances
 class KNewFileMenuSingleton
 {
@@ -93,7 +115,7 @@ public:
         delete templatesList;
     }
 
-    /**
+    /*
      * Opens the desktop files and completes the Entry list
      * Input: the entry list. Output: the entry list ;-)
      */
@@ -120,20 +142,20 @@ public:
     };
     // NOTE: only filePath is known before we call parseFiles
 
-    /**
+    /*
      * List of all template files. It is important that they are in
      * the same order as the 'New' menu.
      */
     typedef QList<Entry> EntryList;
 
-    /**
+    /*
      * Set back to false each time new templates are found,
      * and to true on the first call to parseFiles
      */
     bool filesParsed;
     EntryList *templatesList;
 
-    /**
+    /*
      * Is increased when templatesList has been updated and
      * menu needs to be re-filled. Menus have their own version and compare it
      * to templatesVersion before showing up
@@ -247,118 +269,123 @@ public:
 
     bool checkSourceExists(const QString &src);
 
-    /**
+    /*
      * The strategy used for other desktop files than Type=Link. Example: Application, Device.
      */
     void executeOtherDesktopFile(const KNewFileMenuSingleton::Entry &entry);
 
-    /**
+    /*
      * The strategy used for "real files or directories" (the common case)
      */
     void executeRealFileOrDir(const KNewFileMenuSingleton::Entry &entry);
 
-    /**
+    /*
      * Actually performs file handling. Reads in m_copyData for needed data, that has been collected by execute*() before
      */
     void executeStrategy();
 
-    /**
+    /*
      * The strategy used when creating a symlink
      */
     void executeSymLink(const KNewFileMenuSingleton::Entry &entry);
 
-    /**
+    /*
      * The strategy used for "url" desktop files
      */
     void executeUrlDesktopFile(const KNewFileMenuSingleton::Entry &entry);
 
-    /**
+    /*
      * Fills the menu from the templates list.
      */
     void fillMenu();
 
-    /**
+    /*
      * Tries to map a local URL for the given URL.
      */
     QUrl mostLocalUrl(const QUrl &url);
 
-    /**
+    /*
      * Just clears the string buffer d->m_text, but I need a slot for this to occur
      */
     void slotAbortDialog();
 
-    /**
+    /*
      * Called when New->* is clicked
      */
     void slotActionTriggered(QAction *action);
 
-    /**
+    /*
      * Shows a dialog asking the user to enter a name when creating a new folder.
      */
     void showNewDirNameDlg(const QString &name);
 
-    /**
+    /*
      * Callback function that reads in directory name from dialog and processes it
      */
     void slotCreateDirectory();
 
-    /**
+    /*
      * Fills the templates list.
      */
     void slotFillTemplates();
 
-    /**
+    /*
      * Called when accepting the KPropertiesDialog (for "other desktop files")
      */
     void _k_slotOtherDesktopFile(KPropertiesDialog *sender);
 
-    /**
+    /*
      * Called when closing the KPropertiesDialog is closed (whichever way, accepted and rejected)
      */
     void slotOtherDesktopFileClosed();
 
-    /**
+    /*
      * Callback in KNewFileMenu for the RealFile Dialog. Handles dialog input and gives over
      * to executeStrategy()
      */
     void slotRealFileOrDir();
 
-    /**
+    /*
      * Delay calls to _k_slotTextChanged
      */
     void _k_delayedSlotTextChanged();
 
-    /**
+    /*
      * Dialogs use this slot to write the changed string into KNewFile menu when the user
      * changes touches them
      */
     void _k_slotTextChanged(const QString &text);
 
-    /**
+    /*
      * Callback in KNewFileMenu for the Symlink Dialog. Handles dialog input and gives over
      * to executeStrategy()
      */
     void slotSymLink();
 
-    /**
+    /*
      * Callback in KNewFileMenu for the Url/Desktop Dialog. Handles dialog input and gives over
      * to executeStrategy()
      */
     void slotUrlDesktopFile();
 
-    /**
+    /*
      * Callback to check if a file/directory with the same name as the one being created, exists
      */
     void _k_slotStatResult(KJob *job);
 
     void _k_slotAccepted();
 
-    /**
+    /*
      * Initializes m_fileDialog and the other widgets that are included in it. Mainly to reduce
      * code duplication in showNewDirNameDlg() and executeRealFileOrDir().
      */
     void initDialog();
 
+    /**
+     * Sets the file/folder icon in the new file dialog.
+     */
+    void setIcon(const QIcon &icon);
+
     QAction *m_newFolderShortcutAction = nullptr;
     QAction *m_newFileShortcutAction = nullptr;
 
@@ -368,7 +395,13 @@ public:
     QDialog *m_fileDialog = nullptr;
     KMessageWidget *m_messageWidget = nullptr;
     QLabel *m_label = nullptr;
+    QLabel *m_iconLabel = nullptr;
     QLineEdit *m_lineEdit = nullptr;
+    KCollapsibleGroupBox *m_chooseIconBox = nullptr;
+    QGridLayout *m_folderIconGrid = nullptr;
+    // Exclusive QButtonGroup doesn't allow no button be checked...
+    QActionGroup *m_iconGroup = nullptr;
+    QPushButton *m_chooseIconButton = nullptr;
     QDialogButtonBox *m_buttonBox = nullptr;
 
     // This is used to allow _k_slotTextChanged to know whether it's being used to
@@ -376,13 +409,13 @@ public:
     bool m_creatingDirectory = false;
     bool m_modal = true;
 
-    /**
+    /*
      * The action group that our actions belong to
      */
     QActionGroup *m_newMenuGroup = nullptr;
     QWidget *m_parentWidget = nullptr;
 
-    /**
+    /*
      * When the user pressed the right mouse button over an URL a popup menu
      * is displayed. The URL belonging to this popup menu is stored here.
      * For all intents and purposes this is the current directory where the menu is
@@ -394,6 +427,7 @@ public:
     QStringList m_supportedMimeTypes;
     QString m_tempFileToDelete; // set when a tempfile was created for a Type=URL desktop file
     QString m_text;
+    QString m_windowTitle;
 
     KNewFileMenuSingleton::Entry *m_firstFileEntry = nullptr;
 
@@ -401,7 +435,7 @@ public:
 
     KNewFileMenuCopyData m_copyData;
 
-    /**
+    /*
      * Use to delay a bit feedback to user
      */
     QTimer *m_delayedSlotTextChangedTimer;
@@ -436,22 +470,25 @@ void KNewFileMenuPrivate::initDialog()
     m_fileDialog = new QDialog(m_parentWidget);
     m_fileDialog->setAttribute(Qt::WA_DeleteOnClose);
     m_fileDialog->setModal(m_modal);
-    m_fileDialog->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
-    m_fileDialog->setWindowTitle(i18nc("@title:window", "Create New File"));
 
-    m_messageWidget = new KMessageWidget(m_fileDialog);
-    m_messageWidget->setCloseButtonVisible(false);
-    m_messageWidget->setWordWrap(true);
+    Ui_NewFileDialog ui;
+    ui.setupUi(m_fileDialog);
+
+    m_messageWidget = ui.messageWidget;
+    m_label = ui.label;
+    m_iconLabel = ui.iconLabel;
+    m_lineEdit = ui.lineEdit;
+    m_chooseIconBox = ui.chooseIconBox;
+    m_folderIconGrid = ui.folderIconGrid;
+    m_buttonBox = ui.buttonBox;
+    m_chooseIconButton = ui.chooseIconButton;
+
+    ui.iconHintLabel->setFont(QFontDatabase::systemFont(QFontDatabase::SmallestReadableFont));
+
+    m_iconLabel->hide();
+    m_chooseIconBox->hide();
     m_messageWidget->hide();
 
-    m_label = new QLabel(m_fileDialog);
-
-    m_lineEdit = new QLineEdit(m_fileDialog);
-    m_lineEdit->setClearButtonEnabled(true);
-    m_lineEdit->setMinimumWidth(400);
-
-    m_buttonBox = new QDialogButtonBox(m_fileDialog);
-    m_buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
     QObject::connect(m_buttonBox, &QDialogButtonBox::accepted, [this]() {
         _k_slotAccepted();
     });
@@ -460,15 +497,16 @@ void KNewFileMenuPrivate::initDialog()
     QObject::connect(m_fileDialog, &QDialog::finished, m_fileDialog, [this] {
         m_statRunning = false;
     });
+}
 
-    QVBoxLayout *layout = new QVBoxLayout(m_fileDialog);
-    layout->setSizeConstraint(QLayout::SetFixedSize);
-
-    layout->addWidget(m_label);
-    layout->addWidget(m_lineEdit);
-    layout->addWidget(m_buttonBox);
-    layout->addWidget(m_messageWidget);
-    layout->addStretch();
+void KNewFileMenuPrivate::setIcon(const QIcon &icon)
+{
+    m_iconLabel->setProperty("iconName", icon.name());
+    if (!icon.isNull()) {
+        const QSize iconSize{KIconLoader::SizeHuge, KIconLoader::SizeHuge};
+        m_iconLabel->setPixmap(icon.pixmap(iconSize, m_fileDialog->devicePixelRatioF()));
+    }
+    m_iconLabel->setVisible(!icon.isNull());
 }
 
 bool KNewFileMenuPrivate::checkSourceExists(const QString &src)
@@ -605,6 +643,7 @@ void KNewFileMenuPrivate::executeRealFileOrDir(const KNewFileMenuSingleton::Entr
     }
 
     m_label->setText(entry.comment);
+    setIcon(QIcon::fromTheme(entry.icon));
 
     m_lineEdit->setText(text);
 
@@ -949,6 +988,21 @@ void KNewFileMenuPrivate::slotCreateDirectory()
         KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Mkdir, QList<QUrl>(), url, job);
     }
     job->setProperty("newDirectoryURL", url);
+    if (canPickFolderIcon(url)) {
+        KConfigGroup cfg = stateConfig();
+        cfg.writeEntry("ShowFolderIconPicker", m_chooseIconBox->isExpanded());
+
+        const QString customIconName = m_iconLabel->property("iconName").toString();
+        if (!isDefaultFolderIcon(customIconName)) {
+            job->setProperty("newDirectoryIconName", customIconName);
+
+            QStringList icons = cfg.readEntry("FolderIcons", QStringList());
+            // Move to the end of the list.
+            icons.removeOne(customIconName);
+            icons.append(customIconName);
+            cfg.writeEntry("FolderIcons", icons);
+        }
+    }
     job->uiDelegate()->setAutoErrorHandlingEnabled(true);
     KJobWidgets::setWindow(job, m_parentWidget);
 
@@ -963,8 +1017,18 @@ void KNewFileMenuPrivate::slotCreateDirectory()
 static QStringList getInstalledTemplates()
 {
     QStringList list = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("templates"), QStandardPaths::LocateDirectory);
-    QString templateFolder = QStandardPaths::locate(QStandardPaths::TemplatesLocation, QString(), QStandardPaths::LocateDirectory);
-    list << templateFolder;
+    QString templateFolder = QStandardPaths::standardLocations(QStandardPaths::TemplatesLocation).value(0);
+    static bool templateWarningShown = false;
+    // Some distros set TemplatesLocation to home dir, which means it hasn't been set up and should be ignored
+    // Otherwise everything in the home folder will be used as a template
+    if (templateFolder != QDir::homePath()) {
+        list << templateFolder;
+    } else if (!templateWarningShown) {
+        qCWarning(KFILEWIDGETS_LOG) << "Your 'templates' folder is set to your home folder. "
+                                       "This is probably an error in your settings. Ignoring it. "
+                                       "You can change the setting by running `systemsettings kcm_desktoppaths`. ";
+        templateWarningShown = true;
+    }
     return list;
 }
 
@@ -1223,11 +1287,27 @@ void KNewFileMenuPrivate::_k_slotTextChanged(const QString &text)
 
     // Using a tilde to begin a file or folder name is not recommended
     else if (text.startsWith(QLatin1Char('~'))) {
-        m_messageWidget->setText(
-            i18n("Starting a file or folder name with a tilde is not recommended because it may be confusing or dangerous when using the terminal to delete "
-                 "things."));
-        m_messageWidget->setMessageType(KMessageWidget::Warning);
-        m_messageWidget->animatedShow();
+#ifndef Q_OS_WIN
+        if (KShell::tildeExpand(text).startsWith(QStringLiteral("/home/")) && text.length() > 1) {
+            m_messageWidget->setText(
+                xi18nc("@error",
+                       "Creating a file or folder with the name <filename>%1</filename> is not possible since it would overlap with the location of a user's home folder.",
+                       text,
+                       KShell::tildeExpand(text)));
+            m_messageWidget->setMessageType(KMessageWidget::Error);
+            m_messageWidget->animatedShow();
+            m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+        } else {
+#endif
+            m_messageWidget->setText(i18n(
+                "Starting a file or folder name with a tilde is not recommended because it may be confusing or dangerous when using the terminal to delete "
+                "things."));
+            m_messageWidget->setMessageType(KMessageWidget::Warning);
+            m_messageWidget->animatedShow();
+#ifndef Q_OS_WIN
+            m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
+        }
+#endif
     } else {
         m_messageWidget->hide();
     }
@@ -1484,12 +1564,126 @@ void KNewFileMenuPrivate::showNewDirNameDlg(const QString &name)
 {
     initDialog();
 
-    m_fileDialog->setWindowTitle(i18nc("@title:window", "Create New Folder"));
+    m_fileDialog->setWindowTitle(m_windowTitle.isEmpty() ? i18nc("@title:window", "Create New Folder") : m_windowTitle);
 
-    m_label->setText(i18n("Create new folder in %1:", m_baseUrl.toDisplayString(QUrl::PreferLocalFile)));
+    m_label->setText(i18n("Create new folder in %1:", m_baseUrl.toDisplayString(QUrl::PreferLocalFile | QUrl::StripTrailingSlash)));
 
     m_lineEdit->setText(name);
 
+    const QString defaultFolderIconName = QStringLiteral("inode-directory");
+    setIcon(QIcon::fromTheme(defaultFolderIconName));
+
+    if (canPickFolderIcon(m_baseUrl)) {
+        m_iconGroup = new QActionGroup{m_fileDialog};
+        m_iconGroup->setExclusionPolicy(QActionGroup::ExclusionPolicy::ExclusiveOptional);
+
+        static constexpr int s_folderIconsCount = 2 * 10 - 1; // default icon is always added.
+
+        int x = 0;
+        int y = 0;
+        QStringList icons = {// colors.
+                             // default folder icon goes here.
+                             QStringLiteral("folder-red"),
+                             QStringLiteral("folder-yellow"),
+                             QStringLiteral("folder-orange"),
+                             QStringLiteral("folder-green"),
+                             QStringLiteral("folder-cyan"),
+                             QStringLiteral("folder-blue"),
+                             QStringLiteral("folder-violet"),
+                             QStringLiteral("folder-brown"),
+                             QStringLiteral("folder-grey"),
+                             // emblems.
+                             QStringLiteral("folder-bookmark"),
+                             QStringLiteral("folder-cloud"),
+                             QStringLiteral("folder-development"),
+                             QStringLiteral("folder-games"),
+                             QStringLiteral("folder-mail"),
+                             QStringLiteral("folder-music"),
+                             QStringLiteral("folder-print"),
+                             QStringLiteral("folder-tar"),
+                             QStringLiteral("folder-temp"),
+                             QStringLiteral("folder-important")};
+
+        const QStringList storedFolderIcons = stateConfig().readEntry(QStringLiteral("FolderIcons"), QStringList());
+        for (const QString &icon : storedFolderIcons) {
+            if (!icons.contains(icon)) {
+                icons.append(icon);
+            }
+        }
+
+        while (icons.size() > s_folderIconsCount) {
+            icons.removeFirst();
+        }
+
+        icons.prepend(defaultFolderIconName);
+
+        for (const QString &icon : icons) {
+            const bool isFirstButton = (x == 0 && y == 0);
+
+            auto *action = new QAction{m_iconGroup};
+            action->setIcon(QIcon::fromTheme(icon));
+
+            const QString displayName = isFirstButton ? i18n("Default") : icon;
+            action->setToolTip(displayName);
+            action->setCheckable(true);
+            action->setChecked(isFirstButton);
+
+            auto *button = new QToolButton{m_fileDialog};
+            button->setDefaultAction(action);
+            button->setToolButtonStyle(Qt::ToolButtonIconOnly);
+            button->setIconSize(QSize(KIconLoader::SizeMedium, KIconLoader::SizeMedium));
+
+            m_folderIconGrid->addWidget(button, y, x++);
+            if (x == icons.size() / 2) {
+                x = 0;
+                ++y;
+            }
+        }
+
+        // TODO fix tab order...
+
+        QObject::connect(m_iconGroup, &QActionGroup::triggered, q, [this](QAction *action) {
+            setIcon(action->icon());
+            // We need ExclusiveOptional so that custom icon can have no button checked
+            // but we never want the user uncheck a button manually.
+            action->setChecked(true);
+        });
+        QObject::connect(m_chooseIconButton, &QPushButton::clicked, q, [this] {
+            KIconDialog dialog{m_fileDialog};
+            dialog.setup(KIconLoader::Desktop, KIconLoader::Place);
+            const QString currentIconName = m_iconLabel->property("iconName").toString();
+            if (!isDefaultFolderIcon(currentIconName)) {
+                dialog.setSelectedIcon(currentIconName);
+            }
+            const QString iconName = dialog.openDialog();
+            if (iconName.isEmpty()) {
+                return;
+            }
+
+            if (isDefaultFolderIcon(iconName)) {
+                m_iconGroup->actions().first()->setChecked(true);
+            } else {
+                const auto actions = m_iconGroup->actions();
+                for (auto *action : actions) {
+                    // No break so none are checked when no preset was found.
+                    action->setChecked(action->icon().name() == iconName);
+                }
+            }
+
+            // setChecked does not emit triggered, update the icon manually.
+            if (m_iconGroup->checkedAction()) {
+                setIcon(m_iconGroup->checkedAction()->icon());
+            } else {
+                setIcon(QIcon::fromTheme(iconName));
+            }
+        });
+
+        if (stateConfig().readEntry("ShowFolderIconPicker", false)) {
+            m_chooseIconBox->setExpanded(true);
+        }
+        m_chooseIconBox->show();
+    }
+
     m_creatingDirectory = true;
     _k_slotTextChanged(name); // have to save string in m_text in case user does not touch dialog
     QObject::connect(m_lineEdit, &QLineEdit::textChanged, q, [this]() {
@@ -1556,6 +1750,11 @@ void KNewFileMenu::setSupportedMimeTypes(const QStringList &mime)
     d->m_supportedMimeTypes = mime;
 }
 
+void KNewFileMenu::setWindowTitle(const QString &title)
+{
+    d->m_windowTitle = title;
+}
+
 void KNewFileMenu::slotResult(KJob *job)
 {
     if (job->error()) {
@@ -1573,6 +1772,14 @@ void KNewFileMenu::slotResult(KJob *job)
         // Was this a copy or a mkdir?
         if (job->property("newDirectoryURL").isValid()) {
             QUrl newDirectoryURL = job->property("newDirectoryURL").toUrl();
+            const QString newDirectoryIconName = job->property("newDirectoryIconName").toString();
+
+            // Apply custom folder icon, if applicable.
+            if (!isDefaultFolderIcon(newDirectoryIconName)) {
+                const QUrl localUrl = d->mostLocalUrl(newDirectoryURL);
+                KDesktopFile desktopFile{localUrl.toLocalFile() + QLatin1String("/.directory")};
+                desktopFile.desktopGroup().writeEntry(QStringLiteral("Icon"), newDirectoryIconName);
+            }
             Q_EMIT directoryCreated(newDirectoryURL);
         } else {
             KIO::CopyJob *copyJob = ::qobject_cast<KIO::CopyJob *>(job);
