@@ -35,6 +35,7 @@
 #include <QImage>
 #include <QJsonArray>
 #include <QJsonDocument>
+#include <QMetaMethod>
 #include <QMimeDatabase>
 #include <QObject>
 #include <QPixmap>
@@ -44,6 +45,7 @@
 #include <QTemporaryDir>
 #include <QTemporaryFile>
 #include <QTimer>
+#include <QUuid>
 
 #include <KConfigGroup>
 #include <KFileUtils>
@@ -58,9 +60,19 @@
 
 #include "job_p.h"
 
+#ifdef WITH_QTDBUS
+#include <QDBusConnection>
+#include <QDBusError>
+#include <QDBusReply>
+
+#include "kiofuse_interface.h"
+#endif
+
 namespace
 {
 static qreal s_defaultDevicePixelRatio = 1.0;
+// Time (in milliseconds) to wait for kio-fuse in a PreviewJob before giving up.
+static constexpr int s_kioFuseMountTimeout = 10000;
 }
 
 namespace KIO
@@ -163,8 +175,12 @@ public:
     } currentDeviceCachePolicy = Unknown;
     // the path of a unique temporary directory
     QString m_tempDirPath;
+    // Whether to try using KIOFuse to resolve files. Set to false if KIOFuse is not available.
+    bool tryKioFuse = true;
 
     void getOrCreateThumbnail();
+    void createThumbnailViaFuse(const QUrl &, const QUrl &);
+    void createThumbnailViaLocalCopy(const QUrl &);
     bool statResultThumbnail();
     void createThumbnail(const QString &);
     void cleanupTempFile();
@@ -395,6 +411,8 @@ void PreviewJobPrivate::startPreview()
             item.standardThumbnailer = plugin.description() == QStringLiteral("standardthumbnailer");
             item.plugin = plugin;
             items.push_back(item);
+            bool handlesSequencesValue = item.plugin.value(QStringLiteral("HandleSequences"), false);
+            thumbnailWorkerMetaData.insert(QStringLiteral("handlesSequences"), QString::number(handlesSequencesValue));
 
             if (!bNeedCache && bSave && plugin.value(QStringLiteral("CacheThumbnail"), true)) {
                 const QUrl url = fileItem.targetUrl();
@@ -619,7 +637,10 @@ void PreviewJob::slotResult(KJob *job)
             return;
         }
 
-        d->createThumbnail(static_cast<KIO::FileCopyJob *>(job)->destUrl().toLocalFile());
+        QString pixPath = static_cast<KIO::FileCopyJob *>(job)->destUrl().toLocalFile();
+        if (!pixPath.isEmpty()) {
+            d->createThumbnail(pixPath);
+        }
         return;
     }
     case PreviewJobPrivate::STATE_CREATETHUMB: {
@@ -716,29 +737,84 @@ void PreviewJobPrivate::getOrCreateThumbnail()
         return;
     }
 
-    if (item.isDir()) {
+    if (item.isDir() || !KProtocolInfo::isKnownProtocol(item.targetUrl().scheme())) {
         // Skip remote dirs (bug 208625)
         cleanupTempFile();
         determineNextFile();
         return;
     }
+    // The plugin does not support this remote content, either copy the
+    // file, or try to get a local path using KIOFuse
+    if (tryKioFuse) {
+        createThumbnailViaFuse(item.targetUrl(), item.mostLocalUrl());
+        return;
+    }
+
+    createThumbnailViaLocalCopy(item.mostLocalUrl());
+}
+
+void PreviewJobPrivate::createThumbnailViaFuse(const QUrl &fileUrl, const QUrl &localUrl)
+{
+    Q_Q(PreviewJob);
+#if defined(WITH_QTDBUS) && !defined(Q_OS_ANDROID)
+    state = PreviewJobPrivate::STATE_GETORIG;
+    org::kde::KIOFuse::VFS kiofuse_iface(QStringLiteral("org.kde.KIOFuse"), QStringLiteral("/org/kde/KIOFuse"), QDBusConnection::sessionBus());
+    kiofuse_iface.setTimeout(s_kioFuseMountTimeout);
+    QDBusPendingReply<QString> reply = kiofuse_iface.mountUrl(fileUrl.toString());
+    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, q);
+
+    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, q, [this, localUrl](QDBusPendingCallWatcher *watcher) {
+        QDBusPendingReply<QString> reply = *watcher;
+        watcher->deleteLater();
+
+        if (reply.isError()) {
+            // Don't try kio-fuse again if it is not available
+            if (reply.error().type() == QDBusError::ServiceUnknown || reply.error().type() == QDBusError::NoReply) {
+                tryKioFuse = false;
+            }
+
+            // Fall back to copying the file to the local machine
+            createThumbnailViaLocalCopy(localUrl);
+        } else {
+            // Use file exposed via the local fuse mount point
+            createThumbnail(reply.value());
+        }
+    });
+#else
+    createThumbnailViaLocalCopy(localUrl);
+#endif
+}
+
+void PreviewJobPrivate::createThumbnailViaLocalCopy(const QUrl &url)
+{
+    Q_Q(PreviewJob);
+
+    // Only download for the first sequence
+    if (sequenceIndex) {
+        cleanupTempFile();
+        determineNextFile();
+        return;
+    }
     // No plugin support access to this remote content, copy the file
     // to the local machine, then create the thumbnail
     state = PreviewJobPrivate::STATE_GETORIG;
-    QTemporaryFile localFile;
+    const KFileItem &item = currentItem.item;
 
-    // Some thumbnailers, like libkdcraw, depend on the file extension being
-    // correct
-    const QString extension = item.suffix();
-    if (!extension.isEmpty()) {
-        localFile.setFileTemplate(QStringLiteral("%1.%2").arg(localFile.fileTemplate(), extension));
+    // Build the destination filename: ~/.cache/app/kpreviewjob/pid/UUID.extension
+    QString krun_writable =
+        QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/kpreviewjob/%1/").arg(QCoreApplication::applicationPid());
+    if (!QDir().mkpath(krun_writable)) {
+        qCWarning(KIO_GUI) << "Could not create a cache folder for preview creation:" << krun_writable;
+        cleanupTempFile();
+        determineNextFile();
+        return;
     }
+    tempName = QStringLiteral("%1%2.%3")
+                        .arg(krun_writable)
+                        .arg(QUuid(item.mostLocalUrl().toString()).createUuid().toString(QUuid::WithoutBraces))
+                        .arg(item.suffix());
 
-    localFile.setAutoRemove(false);
-    localFile.open();
-    tempName = localFile.fileName();
-    const QUrl currentURL = item.mostLocalUrl();
-    KIO::Job *job = KIO::file_copy(currentURL, QUrl::fromLocalFile(tempName), -1, KIO::Overwrite | KIO::HideProgressInfo /* No GUI */);
+    KIO::Job *job = KIO::file_copy(url, QUrl::fromLocalFile(tempName), -1, KIO::Overwrite | KIO::HideProgressInfo /* No GUI */);
     job->addMetaData(QStringLiteral("thumbnail"), QStringLiteral("1"));
     q->addSubjob(job);
 }
@@ -844,9 +920,9 @@ void PreviewJobPrivate::createThumbnail(const QString &pixPath)
     bool save = bSave && currentItem.plugin.value(QStringLiteral("CacheThumbnail"), true) && !sequenceIndex;
 
     bool isRemoteProtocol = currentItem.item.localPath().isEmpty();
-    CachePolicy cachePolicy = isRemoteProtocol ? CachePolicy::Prevent : canBeCached(pixPath);
+    currentDeviceCachePolicy = isRemoteProtocol ? CachePolicy::Allow : canBeCached(pixPath);
 
-    if (cachePolicy == CachePolicy::Unknown) {
+    if (currentDeviceCachePolicy == CachePolicy::Unknown) {
         // If Unknown is returned, creating thumbnail should be called again by slotResult
         return;
     }
@@ -901,7 +977,7 @@ void PreviewJobPrivate::createThumbnail(const QString &pixPath)
     job->addMetaData(QStringLiteral("plugin"), currentItem.plugin.fileName());
     job->addMetaData(QStringLiteral("enabledPlugins"), enabledPlugins.join(QLatin1Char(',')));
     job->addMetaData(QStringLiteral("devicePixelRatio"), QString::number(devicePixelRatio));
-    job->addMetaData(QStringLiteral("cache"), QString::number(cachePolicy == CachePolicy::Allow));
+    job->addMetaData(QStringLiteral("cache"), QString::number(currentDeviceCachePolicy == CachePolicy::Allow));
     if (sequenceIndex) {
         job->addMetaData(QStringLiteral("sequence-index"), QString::number(sequenceIndex));
     }
@@ -1008,15 +1084,20 @@ void PreviewJobPrivate::saveThumbnailData(QImage &thumb)
 void PreviewJobPrivate::emitPreview(const QImage &thumb)
 {
     Q_Q(PreviewJob);
-    QPixmap pix;
     const qreal ratio = thumb.devicePixelRatio();
-    if (thumb.width() > width * ratio || thumb.height() > height * ratio) {
-        pix = QPixmap::fromImage(thumb.scaled(QSize(width * ratio, height * ratio), Qt::KeepAspectRatio, Qt::SmoothTransformation));
-    } else {
-        pix = QPixmap::fromImage(thumb);
+
+    QImage preview = thumb;
+    if (preview.width() > width * ratio || preview.height() > height * ratio) {
+        preview = preview.scaled(QSize(width * ratio, height * ratio), Qt::KeepAspectRatio, Qt::SmoothTransformation);
+    }
+
+    Q_EMIT q->generated(currentItem.item, preview);
+
+    if (q->isSignalConnected(QMetaMethod::fromSignal(&PreviewJob::gotPreview))) {
+        QPixmap pixmap = QPixmap::fromImage(preview);
+        pixmap.setDevicePixelRatio(ratio);
+        Q_EMIT q->gotPreview(currentItem.item, pixmap);
     }
-    pix.setDevicePixelRatio(ratio);
-    Q_EMIT q->gotPreview(currentItem.item, pix);
 }
 
 QList<KPluginMetaData> PreviewJob::availableThumbnailerPlugins()
